###什么叫做代理？
顾名思义，代你处理一些东西，举个例子，如果你想买菜：

1.不代理：
    你需要出门前往菜市场，逛，挑，买菜，再拎着回来；
    
2.代理给盒马生鲜：
    你只需要下单，等着他送来。
    
这就是代理，Java设计模式中的官话来说呢，代理就是给某个对象提供一个代理对象，由代理对象控制原对象的引用。
    
###正向代理
听到正向你应该会想到反向代理，我们下面再介绍。

先了解一个流程，我们的主角是<span style="color: orange;">nginx</span>，他跟我们有什么关系呢？一个软件应用由两个部分组成，即客户端和服务端，
本来他们是一对恩爱的夫妻，偏偏nginx要出来插一脚，形成了这样的局面：

    Client <-> Nginx <-> Server
    
那么Nginx起什么作用呢？他在中间捣乱不会影响我们数据传输的效率吗？

例子又来了，请大家李姐。

你平常可能发现自己访问不了Po**hub, youtube等国外的网站，但是人家跟你说挂个VPN就可以啦，你照做，确实可以访问了，
那么VPN是啥呢？
    
引用：<span style="color: orange;">VPN</span> 通俗的讲就是一种中转服务，当我们电脑接入 VPN 后，
我们对外 IP 地址就会变成 VPN 服务器的 公网 IP，我们请求或接受任何数据都会通过这个VPN 服务器然后传入到我们本机。

###反向代理
它和正向代理的区别就是，再引用一句话：

####**<span style="color: orange;">正向代理客户端，反向代理服务器</span>**

正向代理的时候，我们需要修改客户端的配置，所以相当于正向代理是需要自主实现的，而反向代理，客户端是察觉不出来的，他并不知道你是否做了
服务器反向代理。我们只需要发送请求到代理服务器，由代理服务器去跟真实的服务器做通信，再把返回的数据传递给我们。

能感知的：
    
    client -> server ↓
                     ↓
    client  <- server(res)

真实的：
    
    client -> server（proxy） -> server（real）  ↓ 
                                                ↓
    client  <- server(proxy, res) <- server(real, res)
    
例子例子：
 我们平时在本地启动的服务都是通过http://localhost:port来访问的，那么我如果想通过一个网址，如www.abc.com来访问
 http://localhost:port的内容，该怎么处理呢？
 
    给大家介绍一个快速搭建本地服务器的框架，http-server，安装方法：
    
    npm install http-server -g
    
    在一个目录下，直接http-server即可启动，默认端口8080
    
此时我们访问localhost:8080，可以看到


![img](../img/http-server.png)

我们通过修改本地hosts文件，添加一行
    
    127.0.0.1 www.abc.com
    
此时我们就可以通过www.abc.com:8080访问我们刚才的服务页面了
![img](../img/abc-com.png)

此时我们还是需要加上端口号才能正常访问，那我们如果想直接通过www.abc.com访问该怎么做呢？

我们需要先安装nginx，访问http://nginx.org/en/download.html，选择相应的Nginx下载压缩包，解压即可

安装完毕后在conf文件夹下自动生成了nginx.conf文件，内容比较多，我们可以直接修改80端口对应的server配置，产生以下代码

    server {
            listen       80;
            server_name  www.abc.com;
    
            #charset koi8-r;
    
            #access_log  logs/host.access.log  main;
    
            location / {
                proxy_pass http://127.0.0.1:8080;
                root   html;
                index  index.html index.htm;
            }
            
我们再启动nginx，使用cmd工具切换目录到安装目录下

    start nginx即可启动nginx服务
    
这时我们访问www.abc.com就可以看到，其实我们相当于代理了端口号，80为默认端口，所以不加端口号也可以直接访问。跟www.abc.com:80是一样的效果
![img](../img/abc-com-without-port.png)

其中主要字段的含义：
    
###listen: 配置网络监听，主要有以下几种方式：

####1 listen *:80 | *:8080 #监听所有80端口和8080端口
####2 listen  IP_address:port   #监听指定的地址和端口号
####3 listen  IP_address     #监听指定ip地址所有端口
####4 listen port     #监听该端口的所有IP连接

###server_name: 用于虚拟主机的配置：
####1.基于名称的主机配置
一般来说，可以只有一个名称，也可以多个名称通过空格隔开，每个名称由两或者三段组成，由“.”隔开，

    server_name     abc.com www.abc.com;
通过通配符“*”，但该字符只能在三段名称组成的首位或者末位

    server_name     *.abc.com www.acb.*;
也可以使用正则表达式，用“~”作为正则表达式的起始标记

    server_name     ~^www\.abc-.*?\.com$;
上述表达式可以匹配www.abc任意字符任意次数.com的名称

####他们的匹配优先级为：
    1 ①、准确匹配 server_name
    2 ②、通配符在开始时匹配 server_name 成功
    3 ③、通配符在结尾时匹配 server_name 成功
    4 ④、正则表达式匹配 server_name 成功
基于ip地址的虚拟主机配置

    server_name     192.168.1.1
    
###location:  用于匹配url
语法为：

    location [ = | ~ | ~* | ^~ ] url {
    
    }

####1. “=”代表严格匹配url，匹配成功就立即处理
####2. “~”代表匹配带有正则表达式的url，区分大小写
####3. “~*”代表匹配带有正则表达式的url，不区分大小写
####1. “^~”用于不含正则表达式的 uri 前，要求 Nginx 服务器找到标识 uri 和请求字符串匹配度最高的 location 后，立即使用此 location 处理请求，而不再使用 location 块中的正则 uri 和请求字符串做匹配。

###proxy_pass:  用于设置被代理服务器的地址，可以是ip地址，域名等；

    proxy_pass https://www.baidu.com
    
###index:   设置网站的默认首页，可以有多个，空格隔开

    index index.html index.htm index.jsp
